#!/bin/sh
#A proxima linha reinicia usando wish \
    exec wish "$0" "$@"

#"Copyright 2008, 2009, 2010, 2011, 2013 Markos"

# Este programa é um software livre; você pode redistribui-lo e/ou 
# modifica-lo dentro dos termos da GNU LGPL como 
# publicada pela Fundação do Software Livre (FSF); na versão 2 da 
# Licença, ou (na sua opinião) qualquer versão.
#
# Este programa é distribuido na esperança que possa ser  útil, 
# mas SEM NENHUMA GARANTIA; sem uma garantia implícita de ADEQUAÇÂO a qualquer
# MERCADO ou APLICAÇÃO EM PARTICULAR. Veja a
# Licença Pública Geral GNU para maiores detalhes.
#
# Você deve ter recebido uma cópia da Licença Pública Geral GNU
# junto com este programa, se não, escreva para a Fundação do Software
# Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


#Versão 02 do programa orion5star
#Considerando o uso do Starpack, mudamos os comandos para localizar
#a biblioteca objetos.tcl
#Nesta versão foi incluído recursos para o usuário configurar o programa
#para os eletrodos e unidades selecionados na tela do equipamento

#Versão 03 do programa orion5star
#Incluído tela para inserir informações sobre o experimento
#Armazenamento de dados com a biblioteca Metakit

#Versao 00 do programa multipar
#Exibição de gráficos com Plotchart

#Versão 01 do programa multipar
#Inclusão do monitoramento da temperatura usando o sensor
#do medidor de OD

#Versão 02 do programa multipar
#Inclusão de tela inicial para selecionar o modelo do medidor 
#Separar o procedimento ler_porta e tela_config em um arquivo separado para permitir 
#que a manipulação da saída de cada medidor seja feita por uma rotina específica 
#para cada medidor.

#Versão 03 do programa multipar
#Foi necessário fazer modificações no programa principal devido à
#inclusão da variável unidade_tempo para permitir o armazenamento 
#do tempo de cada experimento em diferentes unidades
#As leituras de tempo são armazenadas na respectiva unidade [h|m|s]
#mas optei, por simplicidade, por manter a unidade dos gráficos
#exibidos durante o experimento em minutos.
#por isso incluí na rotina "graficar_medida" um teste condicional
#para fazer o ajuste do tempo para a minutos.
#Isso vai demandar mudanças nos demais módulos para incluir a opção
#de configuração da unidade de tempo

#Versão 00 do programa respirômetro para medidas de Respirometria

#Started using TclOO package for defining classes
#to control the aeration system, sludge addition and pH control
#We use the content of variable lista_par_cont_exp 
#to create objects of class Controller
#We use two lists:
#par_monit - list of parameters "monitored" during experiment
#par_cont - list of parameters "controlled" during experiment
#par_monit_$p
#par_cont_$p

wm title . "Medidor Multi Parâmetro"

#Comando para o gerenciador de janelas para impedir o redimensionamento das janelas

wm resizable . 0 0

#Obtém o diretório atual do script

set dir [file dirname [info script]]

#Adiciona ao path de bibliotecas o diretório
#objLab (Objetos de Laboratório)
lappend auto_path [file join $dir objLab]

#puts $auto_path

#Carrega a biblioteca de Objetos de Laboratorio
#objLab no diretório objLab
package require objLab

#Carrega a biblioteca Metakit para gerenciamento do banco de dados
package require Mk4tcl

#Carrega a biblioteca Plotchart para a exibição de gráficos
package require Plotchart

#Carrega o módulo objCont.tcl

source objcont_05.tcl



#*************************************************************************************
#Criação dos frames principais para acomodar os elementos de interface para cada
#parâmetro, dentro dos quais podem ser criados frames filhos para organizar
#a posição dos elementos gráficos (widgets)
#*************************************************************************************

global fonte_grande fonte_media 

set fonte_grande {Times 18}

set fonte_media {Times 14}

#****************************************************************************
#Seleciona o nome da porta serial conforme o sistema operacional
#no qual o programa estiver rodando
#****************************************************************************

global nome_porta_serial nome_porta_usbserial plataforma

set plataforma $tcl_platform(platform)

if {$plataforma == "unix"} {
        set nome_porta_serial "/dev/ttyS"
	set nome_porta_usbserial "/dev/ttyUSB"
    } else {
        set nome_porta_serial "com"
    }


#****************************************************************************
#Identificação do usuário logado
#A variável resp_exp já existe e armazena o nome digitado na tela de 
#cadastramento do experimento. Por isso vamos usar a variável "usuario_logado"
#Além do array env(USERNAME) também poderíamos usar tcl_platform(user)
#****************************************************************************

set usuario_logado $env(USERNAME)



#*************************************************************************************
#Criação da tela inicial para seleção do modelo de medidor
#************************************************************************************

proc tela_inicial {} {
    
    destroy .cfg

    global fonte_media
    
    set lista_modelos [ listar_arquivos lib_instrument *.tcl ]
    
#    puts $lista_modelos
    
    set frame_modelos [ frame .modelos ]
    
    set mensagem [label $frame_modelos.msg -font $fonte_media -text "Selecione, por favor, \
o modelo do medidor que você está usando!"]
    
    pack $mensagem -pady 8 -padx 10
    
    foreach modulo_tcl $lista_modelos {
	
	set m [ file rootname $modulo_tcl ]
	
	puts $m
	
	radiobutton $frame_modelos.$m -text "$m" -font $fonte_media -relief flat \
	    -variable modelo -value $modulo_tcl
	
	pack $frame_modelos.$m -side top -anchor w
	
    }
    
    set botao [ frame $frame_modelos.bt ]
    
    button $botao.continuar -text "Continuar" -font $fonte_media \
	-command [list carregar_modulo lib_instrument]
    
    button $botao.sair -text "Sair" -font $fonte_media -command "exit"
    
    pack $botao.continuar $botao.sair -side left -expand yes
    
    pack $botao -expand yes -fill x -pady 8
    
    pack $frame_modelos
    
}

#************************************************************************************
#Rotina para a criação de uma lista de arquivos presentes em um diretório
#com um padrão determinado
#************************************************************************************


proc listar_arquivos { pasta padrao } {

cd $pasta

set lista_modelos [ glob -nocomplain $padrao ]

cd ..

return $lista_modelos

}

#************************************************************************************
#Rotina para carregar um módulo
#************************************************************************************

proc carregar_modulo { pasta } {

global modelo

if { $modelo == "" } { 
	tk_messageBox -message "Por favor, selecione um medidor!" 
	return
    }

cd $pasta

source $modelo

set diretorio [pwd]

puts "carregado $modelo da pasta $diretorio"

cd ..

#O frame .modelos está definido no programa principal (multipar_ )
destroy .modelos

#E o frame .janela está definido no módulo do medidor
tela_config .janela

}

#*************************************************************************************
#Criação da tela de monitoramento
#************************************************************************************


proc tela_monit { janela_anterior } {
    
destroy .janela    
    
    global fonte_media fonte_grande porta_serial par_monit
    
    pack forget $janela_anterior
    
puts "par_monit = $par_monit"
    
   
    
    set janela_atual [frame .janela] 
    
    foreach { par nome unidade } $par_monit {
	
	global leituras_dados_$par
	
	
#*************************************************************************************
#Criação dos frames principais para acomodar os elementos de interface para cada
#parâmetro, dentro dos quais podem ser criados frames filhos para organizar
#a posição dos elementos gráficos (widgets)
#Estamos usando a estratégia de dereferenciar, quando uma variável contém o nome
#de outra variável e esta armazena a informação
#Fonte: http://phaseit.net/claird/comp.lang.tcl/tcl_deref.html
#*************************************************************************************
	
	set janela_$par [labelframe $janela_atual.[string tolower $par]  \
	    -text "$nome" -font $fonte_media]
	
	
#*************************************************************************************
#Criação dos frames principais para acomodar os elementos  checkbox e janela de texto
#com as leituras
#*************************************************************************************


	set frame_cb_$par [frame [set janela_$par].selec]
	
#puts "par -> $par - janela_$par -> [set janela_$par] - \[set janela_$par\] -> [set janela_$par]"
	
#*************************************************************************************
#Checkbuttons para ativar ou desativar a aquisição de cada eletrodo individualmente
#*************************************************************************************

#*************************************************************************************
#O procedimento config_label foi um "atalho" para poder reconfigurar o label do checkbox
#*************************************************************************************


	set cb_$par [checkbutton [set frame_cb_$par].cb \
			 -text "Desativado" -font $fonte_media \
			 -variable $par -relief flat \
			 -command [list config_label [set frame_cb_$par].cb $par]]
	
	set frame_dados_$par [frame [set janela_$par].texto]
	
	
#*************************************************************************************
#Janelas de texto para visualizar as leituras
#*************************************************************************************

	set menu_$par [frame [set frame_dados_$par].barramenu -relief raised]
	
	
	set menu_arq_$par [menubutton [set menu_$par].arquivo \
	    -font $fonte_media -text "Arquivo"  -relief raised \
	    -menu [set menu_$par].arquivo.opcoes]
	
	
	menu [set menu_$par].arquivo.opcoes
	[set menu_$par].arquivo.opcoes add command -font $fonte_media \
	    -label "Gravar" \
	    -command " gravar_arquivo_txt [set frame_dados_$par].leituras "
	[set menu_$par].arquivo.opcoes add command -font $fonte_media \
	    -label "Abrir" \
	    -command " abrir_arquivo  [set frame_dados_$par].leituras "
	
#puts "par -> $par"
#puts "frame_dados_\$par	-> frame_dados_$par"

	set leituras_dados_$par [text [set frame_dados_$par].leituras -relief sunken -yscrollcommand "[set frame_dados_$par].rolagem set" -background white -width 30 -height 5]
	
#puts [set leituras_dados_$par]

	set rolagem_dados_$par [scrollbar [set frame_dados_$par].rolagem -command "[set frame_dados_$par].leituras yview"]
	
	
	pack [set cb_$par] -side left -expand yes -anchor w
	
	pack [set menu_$par] -side top -fill x
	
	pack [set menu_arq_$par] -fill x 
	
	pack [set leituras_dados_$par] [set rolagem_dados_$par] -side left -expand yes  -fill both
	
	pack [set frame_cb_$par] [set frame_dados_$par] -side left -expand yes -fill both
	
	pack [set janela_$par] -anchor w -pady 8 -expand yes -fill both 
	
}
    
    
     
    #***********************************************************************
    #Criação dos botões para controle do programa
    #Usamos os colchetes e ";" para permitir a substituição de variáveis
    #nos procedimentos incluídos na opção command
    #***********************************************************************
    
set janela_botoes [frame $janela_atual.b]
    
button  $janela_botoes.iniciar -font $fonte_media -text "Iniciar"  -command " [list iniciar_leitura $porta_serial] ; \
[list $janela_botoes.iniciar configure -state disable]; \
    [list $janela_botoes.configurar configure -state disable]; \
exibir_tela_grafico "


button $janela_botoes.parar -font $fonte_media -text "Parar" \
-command [list alerta_parar_leitura $janela_botoes]


#button $janela_botoes.sair -font $fonte_media -text "Sair"  -command { 
#    parar_leitura
#    exit
#}

button $janela_botoes.configurar -font $fonte_media -text "Voltar" -command " [list voltar_config $janela_atual] "


pack $janela_botoes.iniciar $janela_botoes.parar $janela_botoes.configurar \
    -side left -expand yes -fill x

pack $janela_botoes -side bottom -expand yes -fill both

pack $janela_atual

}

###########################################################################
#Procedimento para alertar que parada de leitura vai parar também um 
#experimento
###########################################################################

proc alerta_parar_leitura { janela_botoes } {
    
    global estado_experimento  
    
    
    if { [info exists estado_experimento] } {
	    
#	    puts "Dentro de alerta_parar_experimento e estado_experimento => $estado_experimento"
	    
	    if { $estado_experimento } {
		    
		    set resposta [tk_dialog .parar_leitura "Parada do Experimento"  \
			"Se interromper as leituras vai parar também o experimento que está em andamento! \n \
Deseja Continuar ou Parar o Experimento?"  info 0 "Continuar Exp." "Parar Exp."]
		    switch $resposta {
			0 {}
			1 {
			    parar_leitura
			    $janela_botoes.iniciar configure -state normal
			    $janela_botoes.configurar configure -state normal
			    parar_experimento
			}    
		    }
		} else {
		    parar_leitura
		    $janela_botoes.iniciar configure -state normal
		    $janela_botoes.configurar configure -state normal
		}	
		
	}
	
}


#*************************************************************************************
#Um "remendo" para poder reconfigurar o label do checkbox
#*************************************************************************************


proc config_label { cb par } {
    
    global $par
    if {[set $par]} {
	$cb configure -text "  Ativado   "
    } else {
	$cb configure -text "Desativado"
    }
    
}


#***********************************************************************
#Procedimento criado para gravar os conteúdo de um "text" em um arquivo.
#***********************************************************************

proc gravar_arquivo_txt { janela } {
    
    set dados [$janela get 1.0 {end -1c}]
    set tipo_arquivo {
	{"Arquivos de Texto"    { .txt } }
	{"Todos os arquivos" * }
    }
    set nome_arquivo [tk_getSaveFile -filetypes $tipo_arquivo -initialdir pwd]
    if {[catch {set id_arquivo [open $nome_arquivo w]} result] != 0} {
	return
    }
    puts -nonewline $id_arquivo $dados
    close $id_arquivo
}



#****************************************************************************
#Procedimento criado para abrir os conteúdo de um arquivo em uma janela text.
#****************************************************************************

proc abrir_arquivo { janela } {
    set tipo_arquivo {
	{"Arquivos de Leituras" { .csv } }
	{"Arquivos de Texto"    { .txt } }
	{"Arquivos de Eventos" { .log } }
	{"Todos os arquivos" * }
    }
    
    if {[catch {set nome_arquivo [tk_getOpenFile -filetypes $tipo_arquivo -initialdir pwd]} result] !=0 } {
	return
    }
    if {[catch {set id_arquivo [open $nome_arquivo r]} result] != 0 } {
	return
    }
    set dados [read $id_arquivo ]
    close $id_arquivo
    $janela delete 1.0 end
    $janela insert end $dados
}



#***********************************************************************
#Procedimento criado para abrir a porta serial
#Neste procedimento podemos incluir os parâmetros de comunicação 
#do equipamento
#***********************************************************************




proc iniciar_leitura { porta_fisica } {
    
    global t_inicial
    global estado_experimento

    global list_t_interval

    set estado_experimento 0
    
    cadastrar_experimento

    set t_inicial [clock clicks -milliseconds]
    
#puts "t_inicial $t_inicial"
    
    set id_porta [abrir_porta $porta_fisica]

#puts "id_porta = $id_porta"
    if {$id_porta != "SERIAL_ERRO_OPEN"} {
	fileevent $id_porta readable [list ler_porta $id_porta]
    }

set list_t_interval {}
    
puts "Iniciar Leitura list_t_interval:$list_t_interval"

}


proc parar_leitura { } {

global id_porta id_after_monitconnection

#puts "comando catch retorna [catch {tell $id_porta}]"

if {![catch {tell $id_porta}]} {
   
#puts "porta $id_porta fechada"
close $id_porta
set id_porta {}

}

puts "Cancel schedulling of monitconnection id_after:$id_after_monitconnection"
after cancel $id_after_monitconnection

return

}



#****************************************************************************
#Cria (ou remove) os objetos da classe sensor conforme os checkboxes
#são ativados ou desativados na interface.
#Este procedimento será chamado toda vez que a variável pH, COND, OD ou TEMP 
#for modificada, vinculação criada pelo comando "trace"
#No momento da criação do objeto eletrodo aproveitamos para definir a
#unidade de leitura que corresponde ao segundo elemento da lista unidade
#****************************************************************************

proc inst { args } {
    
    upvar \#0 [lindex $args 0] par
    set nome_par [lindex $args 0]
    set nome_unid_par [lindex $args 1]

    global frame_cb_$nome_par.cb
    
    if { $par } {
	
	puts "criando sensor eletrodo_$nome_par e \$par : $par"
	puts "Unidade: [lindex $nome_unid_par end]"
	sensor eletrodo_$nome_par
	eletrodo_$nome_par def_unidade [lindex $nome_unid_par end]
	
    } else {
	
	puts "removendo sensor eletrodo_$nome_par e \$par : $par"
	remover_sensor eletrodo_$nome_par
	
    }
}






##############################################################################
#Procedimento para criação de um objeto da classe "medida"
##############################################################################

proc iniciar_medida {parametro unid_tempo leitura} {
    

    global codigo_amostra t_inicial_exp
 
#Array no formato lista_medidas(parametro) para armazenar as leituras de um 
#experimento

    global lista_medidas t_inicial_exp

#Verifica se o parâmetro está sendo monitorado neste experimento    
    set pos_par_lista_par_monit \
	[lsearch [get_parametros_monitorados $codigo_amostra] $parametro]	

    if { ($pos_par_lista_par_monit != -1) } {

#Comando para calcular o tempo atual desde o início do experimento
#em milisegundos   

	    set t_final_exp [clock clicks -milliseconds]

#Testar a unidade de tempo para calcular o tempo correspondente
if {$unid_tempo == "h"} {
	
	#Comando para calcular o tempo (em horas) decorrido desde t0
	set t_final_exp [format "%.4f" [expr ($t_final_exp - $t_inicial_exp)/3600000.0]]
	
    } elseif {$unid_tempo == "m"} {
	
	#Comando para calcular o tempo (em minutos) decorrido desde t0
	set t_final_exp [format "%.2f" [expr ($t_final_exp - $t_inicial_exp)/60000.0]]
	
    } else {

	#Comando para calcular o tempo (em minutos) decorrido desde t0
	set t_final_exp [format "%.1f" [expr ($t_final_exp - $t_inicial_exp)/1000.0]]
    }

   
	    eletrodo_$parametro def_leitura [list $t_final_exp $leitura]
	    
#################################################################
#Comando para a criação de um objeto da classe medida cujo
#nome é o próprio nome do parâmetro monitorado
#################################################################
	    

	    medida medida_$parametro

#puts "iniciar_medida linha 4: [expr ([clock clicks -milliseconds] - $t_inicial_exp)/60000.0]]"
	    
	    medida_$parametro def_amostra_medida $codigo_amostra
	    

	    medida_$parametro def_parametro_medido $parametro


	    medida_$parametro def_parametro_unidade \
		[eletrodo_$parametro get_unidade]

#	    lappend lista_medidas($parametro) \
#		[lindex [eletrodo_$parametro get_leitura] 0]

#	    lappend lista_medidas($parametro) \
#		[lindex [eletrodo_$parametro get_leitura] 1]

# puts "LISTA_MEDIDAS => $lista_medidas($parametro)"
#	    medida_$parametro def_leituras $lista_medidas($parametro)


	    medida_$parametro def_leituras [eletrodo_$parametro get_leitura]
	    

#	    puts "parametro $parametro t_final_exp $t_final_exp unid_tempo $unid_tempo leitura $leitura unid_leitura [eletrodo_$parametro get_unidade]\n"
#puts [eletrodo_$parametro get_leitura]
#puts [$codigo_amostra get_projeto]	    
#puts [$codigo_amostra get_experimento]
#puts [$codigo_amostra get_responsavel_experimento]
#puts [$codigo_amostra get_coleta]    
#puts [clock format [lindex [$codigo_amostra get_coleta] 1] \
#	    -format {%d/%m/%Y %H:%M}]
#puts [$codigo_amostra get_parametros_monitorados]
	    	    
	        
#puts "Código da amostra medida [medida_$parametro get_amostra_medida]"
#puts "Parâmetro monitorado [medida_$parametro get_parametro_medido]"
#puts "Unidade do parâmetro monitorado [medida_$parametro get_parametro_unidade]"	    
#puts "****LEITURAS REALIZADAS [medida_$parametro get_leituras]****"

#Procedimento para arquivar a medida realizada 

	    arquivar_medida medida_$parametro

#Procedimento para exibir a medida no respectivo gráfico

	    graficar_medida medida_$parametro 	    

#Chama o procedimento updateReadings do módulo objCont para permitir
#o controle do parâmetro

	    updateReadings $parametro $t_final_exp $leitura

#Método para remover da memória o objeto da Classe "medida"

	    remover_medida medida_$parametro


	}

	
	return
	
}


##############################################################################
#Procedimento para cadastrar os dados do experimento
##############################################################################

proc cadastrar_experimento { } {
    
    
    destroy .id_exp
    toplevel .id_exp
    
    wm title .id_exp "Cadastrar Experimento"
    
    wm resizable .id_exp 0 0

set tam_entry 15
    
    global fonte_media
    global cod_proj coord_proj num_exp resp_exp local_col resp_col 
    global data_coleta_dia data_coleta_mes data_coleta_ano data_coleta_hora data_coleta_min
    global par_monit
    global estado_experimento
   
    message .id_exp.msg  -justify left -width [expr 60 * $tam_entry]  -font $fonte_media -text "Por favor, forneça as informações sobre o experimento. Obrigado!"
    
    pack .id_exp.msg -expand yes -fill both
    
    #
    #Este loop cria os frames do tipo labelframe contendo as entradas para as informações
    #sobre o experimento
#
    
    frame .id_exp.info
    
    labelframe .id_exp.info.proj -font $fonte_media -text "Projeto:"
  
    label .id_exp.info.proj.cod_proj_l -font $fonte_media -text "Código:"
    entry .id_exp.info.proj.cod_proj_e -width $tam_entry -bg white -textvariable cod_proj
    pack .id_exp.info.proj.cod_proj_l .id_exp.info.proj.cod_proj_e -side left \
	-expand yes -fill both 

    label .id_exp.info.proj.coord_proj_l -font $fonte_media -text "    Coordenador:"
    entry .id_exp.info.proj.coord_proj_e -width $tam_entry -bg white -textvariable coord_proj
    pack .id_exp.info.proj.coord_proj_l .id_exp.info.proj.coord_proj_e -side left \
	-expand yes -fill both

    pack .id_exp.info.proj -expand yes -fill both -pady 8
    
    
    labelframe .id_exp.info.exp -font $fonte_media -text "Experimento:"
    
    label .id_exp.info.exp.num_exp_l -font $fonte_media -text "Número:"
    entry .id_exp.info.exp.num_exp_e -width $tam_entry -bg white -textvariable num_exp
    pack .id_exp.info.exp.num_exp_l .id_exp.info.exp.num_exp_e -side left \
	-expand yes -fill both
    
    label .id_exp.info.exp.resp_exp_l -font $fonte_media -text "    Responsável:"
    entry .id_exp.info.exp.resp_exp_e -width $tam_entry -bg white -textvariable resp_exp
    pack .id_exp.info.exp.resp_exp_l .id_exp.info.exp.resp_exp_e -side left \
	-expand yes -fill both
    
    pack .id_exp.info.exp -expand yes -fill both -pady 8


    labelframe .id_exp.info.col -font $fonte_media -text "Coleta da Amostra:"
    
    label .id_exp.info.col.local_col_l -font $fonte_media -text "Local:"
    entry .id_exp.info.col.local_col_e -width $tam_entry -bg white -textvariable local_col
    pack .id_exp.info.col.local_col_l .id_exp.info.col.local_col_e -side left \
	-expand yes -fill both
    
    label .id_exp.info.col.resp_col_l -font $fonte_media -text "    Responsável:"
    entry .id_exp.info.col.resp_col_e -width $tam_entry -bg white -textvariable resp_col
    pack .id_exp.info.col.resp_col_l .id_exp.info.col.resp_col_e -side left \
	-expand yes -fill both
    
    pack .id_exp.info.col -expand yes -fill both -pady 8


    
    #Campos para entrada de data e hora de amostragem
    #Vão armazenar as variáveis:  data_coleta_dia,  data_coleta_mes e  data_coleta_ano
    #contendo respectivamente o dia mes e ano da amostragem
    
labelframe .id_exp.info.data_hora -font $fonte_media -text "Data e Hora da Coleta"

    frame .id_exp.info.data_hora.data 
       
    label .id_exp.info.data_hora.data.d_l -font $fonte_media -text "Dia"
    spinbox .id_exp.info.data_hora.data.d  -from 1 -to 31  -validate key \
	-vcmd {string is integer %P} -width 2 -bg white \
	-font $fonte_media -textvariable  data_coleta_dia
    
    label .id_exp.info.data_hora.data.m_l -font $fonte_media -text "Mes"
    spinbox .id_exp.info.data_hora.data.m  -from 1 -to 12  -validate key \
	-vcmd {string is integer %P} -width 2 -bg white \
	-font $fonte_media -textvariable  data_coleta_mes
    
    label .id_exp.info.data_hora.data.a_l -font $fonte_media -text "Ano"
    spinbox .id_exp.info.data_hora.data.a  -from 2009 -to 2050  -validate key \
	-vcmd {string is integer %P} -width 4 -bg white \
	-font $fonte_media -textvariable  data_coleta_ano


    grid .id_exp.info.data_hora.data.d_l  .id_exp.info.data_hora.data.m_l  \
	.id_exp.info.data_hora.data.a_l
    grid .id_exp.info.data_hora.data.d  .id_exp.info.data_hora.data.m  \
	.id_exp.info.data_hora.data.a
    

    frame .id_exp.info.data_hora.hora
   

    label .id_exp.info.data_hora.hora.h_l -font $fonte_media -text "Hora-24h"
    spinbox .id_exp.info.data_hora.hora.h  -from 0 -to 23  -validate key \
	-vcmd {string is integer %P} -width 2 -bg white \
	-font $fonte_media -textvariable  data_coleta_hora

    label .id_exp.info.data_hora.hora.min_l -font $fonte_media -text "Min."
    spinbox .id_exp.info.data_hora.hora.min  -from 0 -to 60  -validate key \
	-vcmd {string is integer %P} -width 2 -bg white \
	-font $fonte_media -textvariable  data_coleta_min

    grid .id_exp.info.data_hora.hora.h_l  .id_exp.info.data_hora.hora.min_l
    grid .id_exp.info.data_hora.hora.h  .id_exp.info.data_hora.hora.min

 pack  .id_exp.info.data_hora.data .id_exp.info.data_hora.hora  -side left



pack .id_exp.info.data_hora  -pady 8 


#Frame que armazena os parâmetros monitorados com os respectivos limites (sup e inf)

labelframe .id_exp.info.par_monit -font $fonte_media -text "Parâmetros Monitorados"
pack  .id_exp.info.par_monit

set c 0

    foreach {p n u} $par_monit {
	
	checkbutton .id_exp.info.par_monit.[string tolower $p] -text "$n" -font $fonte_media -variable par_monit_$p 
	label .id_exp.info.par_monit.label_lim_sup_[string tolower $p] -font $fonte_media -text "Limite Superior"
	entry .id_exp.info.par_monit.entry_lim_sup_[string tolower $p] -width 10 -bg white -textvariable lim_sup_$p
	
	label .id_exp.info.par_monit.label_lim_inf_[string tolower $p] -font $fonte_media -text "Limite Inferior"
	entry .id_exp.info.par_monit.entry_lim_inf_[string tolower $p] -width 10 -bg white -textvariable lim_inf_$p

	grid .id_exp.info.par_monit.[string tolower $p] -row 0 -column $c -columnspan 2
	grid .id_exp.info.par_monit.label_lim_sup_[string tolower $p] -row 1 -column $c
	grid .id_exp.info.par_monit.entry_lim_sup_[string tolower $p] -row 1 -column [expr $c + 1]

	grid .id_exp.info.par_monit.label_lim_inf_[string tolower $p] -row 2 -column $c
	grid .id_exp.info.par_monit.entry_lim_inf_[string tolower $p] -row 2 -column [expr $c + 1]

	incr c 2
    } 
    
   

    
    labelframe .id_exp.coment -font $fonte_media -text "Comentários:"
    text .id_exp.coment.txt -width 30 -height 5 \
	-yscrollcommand { .id_exp.coment.rlg set} -background white
    scrollbar .id_exp.coment.rlg -command { .id_exp.coment.txt yview }
    pack .id_exp.coment.txt -side left -expand yes -fill both
    pack .id_exp.coment.rlg -side right -fill y
    
    frame .id_exp.b
    
    button .id_exp.b.continuar -font $fonte_media -text "Iniciar Experimento" \
	-command iniciar_experimento

    button .id_exp.b.parar -font $fonte_media -text "Parar Experimento" \
	-command parar_experimento

    button .id_exp.b.consultar_db -font $fonte_media \
	-text "Banco de Dados" -command consultar_cadastro

    button .id_exp.b.cancelar -font $fonte_media \
	-text "Fechar" -command { destroy .id_exp }
    
    pack .id_exp.b.continuar .id_exp.b.parar .id_exp.b.consultar_db .id_exp.b.cancelar -side left -expand yes -fill both
    
    pack .id_exp.info .id_exp.coment  .id_exp.b -expand yes -fill both -side top


}

##############################################################################
#Procedimento para iniciar um experimento
#Este procedimento realiza várias ações:
#1-reinicializa a variável global array lista_medidas(parametro) que armazena
#as leituras de cada parâmetro
#2-inicializa a lista par_cont  com os parâmetros que serão
#monitorados neste experimento e os respectivos limites sup e inf
#Inicializa as seguintes variáveis:
#cod_proj => código do projeto
#coord_proj => coordenador do projeto
#num_exp => número do experimento
#resp_exp => responsável pelo experimento
#local_col => local da coleta
#data_col => data da coleta (format epoch)
#resp_col => responsável pela coleta
#data_coleta_dia
#data_coleta_mes
#data_coleta_ano
#data_coleta_hora
#data_coleta_min
#Cria um objeto "amostra" e configura os respectivos atributos
#Inicializa a variável t_inicial_exp
###############################################################################


proc iniciar_experimento {} {
    
    global t_inicial_exp
    global estado_experimento
    global codigo_amostra
    global par_monit
    global cod_proj coord_proj num_exp resp_exp local_col resp_col
    global data_coleta_dia data_coleta_mes data_coleta_ano data_coleta_hora data_coleta_min
    
#Modelo do medidor e usuário logado
    global modelo usuario_logado
    
#Array no formato lista_medidas (parametro) para armazenar as leituras de um experimento
    
    global lista_medidas
    
#Comando para limpar o conteúdo da variável lista_medidas do experimento anterior

    if { [info exists lista_medidas] } { unset lista_medidas }
	
	set par_cont {}
	
	set lista_par_monit {}
	
	foreach {p n u} $par_monit {
	    
	    global par_monit_$p lim_sup_$p lim_inf_$p
	    
	    lappend lista_par_monit $p 
	    lappend lista_par_monit [set lim_sup_$p]
	    lappend lista_par_monit [set lim_inf_$p]
	    
	    if { [set par_monit_$p] } {
		    lappend par_cont  $p
		    lappend par_cont   [set lim_sup_$p]
		    lappend par_cont   [set lim_inf_$p]
		    
		}
		
	}


	if {![llength $par_cont  ]} {
		tk_messageBox \
		    -message "Por favor, selecione pelo menos \"1\" parâmetro para controle!" \
		    -parent .id_exp
		return
	    }	
	    
	
	.id_exp.b.continuar configure -state disable
	.id_exp.b.cancelar configure -state disable
	
	set t_inicial_exp [clock clicks -milliseconds]
	
	set codigo_amostra [clock seconds]	    
	
	
#Usuário logado no sistema
	
	.id_exp.coment.txt insert end "Usuário logado: $usuario_logado\n"
	
#Modelo do medidor em uso
	
	.id_exp.coment.txt insert end "Medidor: [file rootname $modelo]\n"
	
	
#Aproveitando o conteúdo da variável codigo_amostra para registrar o início do experimento
	set data_hora_inicio_exp [clock format $codigo_amostra -format "%d/%m/%y - %H:%M:%S"]
	
	.id_exp.coment.txt insert end "\nInício do experimento: $data_hora_inicio_exp\n"

	set estado_experimento 1
		
#Criação de um objeto da classe amostra cujo nome é a hora
#local no formato epoch
	
	amostra $codigo_amostra
	
#par_cont armazena a lista contendo os parâmetros controlados no experimento
#atual com os respectivos limites sup e inf
#e lista_par_monit é apenas a lista de todos os sensores ativos no formato
#par lim_sup lim_inf
	
	puts "Variável par_monit:$par_monit"
	puts "Variável lista_par_monit: $lista_par_monit"
	puts "Variável par_cont:$par_cont"
	

	    
	    $codigo_amostra def_parametros_monitorados $lista_par_monit  
	    
	    $codigo_amostra def_projeto $cod_proj $coord_proj 
	    
	    $codigo_amostra def_experimento $num_exp andamento
	    
	    $codigo_amostra def_responsavel_experimento $resp_exp 
	    
	    
#Comando que passou a apresentar problemas quando mudei da versão 8.5 para 8.4
##    set data_col [clock scan "$data_coleta_dia $data_coleta_mes \
##	$data_coleta_ano $data_coleta_hora $data_coleta_min" \
##	-format {%d %m %Y %H %M}]

#Inclui as barras e mudei a posição da variável mes pela variável dia 
#Para ficar de acordo com o padrão americano e evitar erro
#Essa alteração foi necessária após mudar o interpretador Tcl da versão
#8.4 para a versão 8.5

set data_col [clock scan "$data_coleta_mes/$data_coleta_dia/$data_coleta_ano $data_coleta_hora:$data_coleta_min"]

#puts "Data coleta $data_col"

#Armazena as informações sobre a coleta {local data responsavel}

$codigo_amostra def_coleta $local_col $data_col $resp_col

#Procedimento para arquivar os dados cadastrais do experimento

arquivar_experimento $codigo_amostra

#calls the procedure configControl defined in module objcont
#which will create objects of class controller and take other actions
#returns 1 if all is OK, otherwise returns 0.

if {$par_cont != ""} {
	if {[configControl $par_cont]} {
		.id_exp.coment.txt insert end "Iniciando Experimento COM controle dos parâmetros $par_cont"
	    } else {
		.id_exp.coment.txt insert end "Problemas para controle dos parâmetros $par_cont."
	    }
	    
    }


    
}
    
##############################################################################
#Procedimento para parar o experimento o qual vai executar as seguintes ações:
#1-atribuir à variável estado_experimento o valor "0"
#2-arquivar o conteúdo do campo comentários no banco de dados consultando
#o registro da tabela "cadastro" usando o conteúdo da variável num_cad_exp
#3-habilitar os botões "Iniciar Experimento" "Fechar" da tela de cadastramento
#4-remover o objeto "amostra"
#Percebi a necessidade de criar a variável global num_exp_ant
#para controlar o símbolo que deve ser usado na sobreposição de dados
#na rotina graficar_medida
############################################################################## 
    
    
    proc parar_experimento {{msg Normal}} {
	
	global estado_experimento
	global codigo_amostra
	global reg_cad_exp
	global num_exp num_exp_ant
	
if { $estado_experimento } {    

	    set estado_experimento 0
	    
	    set data_hora_final_exp [clock seconds]
	    set data_hora_final_exp [clock format $data_hora_final_exp \
		-format "%d/%m/%y - %H:%M:%S"]

	    .id_exp.coment.txt insert end "\nFinal do experimento: $data_hora_final_exp"

	    .id_exp.coment.txt insert end "\nStatus: $msg\n"

	    set coment_exp [.id_exp.coment.txt get 1.0 {end -1c}]

    
	    mk::file open db medidor_multiparametro.db
	    
#puts "parar_experimento coment_exp:$coment_exp e reg_cad_exp:$reg_cad_exp"

#Necessário incluir aspas na variável "$coment_exp" para ser armazenada
#corretamente    

	    mk::set $reg_cad_exp coment_exp "$coment_exp"
	    
	    mk::file commit db
	    
	    mk::file close db
	    
	    .id_exp.b.continuar configure -state normal
	    .id_exp.b.cancelar configure -state normal
	    
	    remover_amostra $codigo_amostra 

	    set num_exp_ant $num_exp

	    mudar_simbolo

#Chama o procedimento stopControl do módulo objCont para permitir
#o controle do parâmetro
	    stopControl 
    }

}


##############################################################################
#Procedimento para arquivar as informações do cadastro do experimento no 
#banco de dados:
#1-Cria o banco de dados se ele ainda não existe
#2-Resgata os atributos do objeto do tipo amostra e armazena no array exp
#3-Arquiva o conteúdo do array "exp" no banco de dados e guarda a localização
#do registro na variável reg_cad_exp que será usada pelo procedimento 
#parar_experimento para armazenar o conteúdo do campo "comentários" 
##############################################################################

proc arquivar_experimento { info_amostra } {
    
    global reg_cad_exp   
    global num_id_cad_exp
    
#Array no formato reg_cad_med(parametro) para armazenar o número do
#registro na tabela de resultados "db.medida" para agilizar o arquivamento
#das medidas

    global reg_cad_med 


	if {![file exists "medidor_multiparametro.db"]} {
		
		criar_db
	    }
	    
	    
	    #puts "Projeto(código e coordenador): [$info_amostra get_projeto]"
	    set exp(cod_proj) [lindex [$info_amostra get_projeto] 0]
	    set exp(coord_proj) [lindex [$info_amostra get_projeto] 1] 
	    
	    #puts "Experimento (número status): [$info_amostra get_experimento]"
	    set exp(num_exp) [lindex [$info_amostra get_experimento] 0]
	    
	    #puts "Responsável Experimento: \
[$info_amostra get_responsavel_experimento]"
	    set exp(resp_exp) [$info_amostra get_responsavel_experimento]
	    
	    set exp(local_col) [lindex [$info_amostra get_coleta] 0]
	    set exp(data_col)  [lindex [$info_amostra get_coleta] 1]
	    set exp(resp_col)  [lindex [$info_amostra get_coleta] 2]
	    #puts "Coleta(local data resp): [$info_amostra get_coleta]"
	    
	    #puts "Data Coleta: [clock format \
[lindex [$info_amostra get_coleta] 1] -format {%d/%m/%Y %H:%M}]"
	    
	    
	    set exp(lst_par_monit_amostra) [$info_amostra get_parametros_monitorados] 
	    #puts "Parâmetros monitorados(p l_inf l_sup): \
#[$info_amostra get_parametros_monitorados]"
	    
	    
	    
	    
	    mk::file open db medidor_multiparametro.db
	    
	    set exp(id_cad_exp) [getuniqueid db]
	    set num_id_cad_exp $exp(id_cad_exp)

	    #puts "SAÍDA DO ARRAY GET **EXP** [array get exp]"
	    
	    set reg_cad_exp \
		[eval mk::row append db.cadastro [array get exp]]
	    puts "RETORNO DA INCLUSÃO NO CADASTRO DE EXPERIMENTO => $reg_cad_exp"

	    mk::file commit db
	    
	    mk::file close db
	    
	    	    
}

###########################################################################
#Procedimento para criação do banco de dados no arquivo medidor_multiparametro.db
#É executado apenas quando o arquivo medidor_multiparametro.db não existe
###########################################################################


proc criar_db {} {
    
#puts "CRIANDO O BANCO DE DADOS MEDIDOR_MULTIPARAMETRO.DB"

    mk::file open db medidor_multiparametro.db

#Criação da tabela para armazenar os dados cadastrais de um experimento
    
    set cad_exp [mk::view layout db.cadastro {id_cad_exp:L cod_proj:S \
	coord_proj:S num_exp resp_exp:S local_col:S data_col:L resp_col:S \
	lst_par_monit_amostra coment_exp}]
    
#Criação da tabela para armazenar as leituras de um parâmetro durante
#um experimento onde o campo id_cad_med será correspondente ao
#id_cad_exp no qual a medida foi gerada


    
    set cad_med [mk::view layout db.medida {id_cad_med:L parametro:S tempo:F unid_tempo:S leitura:F unid_leitura}]
  
    set controle [mk::view layout db.cad_control {nextid:L}]
  
    mk::row append db.cad_control nextid 1

    puts "cad_exp => $cad_exp, cad_med => $cad_med e controle => $controle"
    
    mk::file close db

    
}

###########################################################################
#Procedimento para geração de chave primária para indexação dos registros
###########################################################################


proc getuniqueid { db } {
    
    set id [mk::get db.cad_control!0 nextid]
    
    mk::set db.cad_control!0 nextid [expr {$id + 1 }]
    
    mk::file commit db
    
    return $id
}


###########################################################################
#Procedimento para arquivar as medidas de um experimento
#Este procedimento anexa novos registros na tabale db.medida
#com os pares {tempo leitura ...} no formato de ponto decimal
#para otimizar o armazenamento e o seu uso em cálculos
##Após arquivar a medida no banco de dados exibe a leitura
#no respectivo gráfico
###########################################################################

proc arquivar_medida { info_medida } {
    
    global num_id_cad_exp unidade_tempo

    set id_cad_med $num_id_cad_exp
    
    set parametro [$info_medida get_parametro_medido]
    
    set tempo  [lindex [lindex [$info_medida get_leituras] 0 ] 0 ]

#puts "info_medida => $info_medida"
#puts "\[\$info_medida get_leituras\] => [$info_medida get_leituras]"
#puts "\[lindex \[\$info_medida get_leituras\] 0 \] => [lindex [$info_medida get_leituras] 0 ]"

    set unid_tempo $unidade_tempo

    set leitura [lindex [lindex [$info_medida get_leituras] 0] 1 ]

    set unid_leitura [$info_medida get_parametro_unidade]

#puts "Dentro de arquivar medida \n\
#parametro  =>$parametro\n\
#tempo      =>$tempo    \n\
#unid_tempo =>$unid_tempo\n\
#leitura    =>$leitura  \n\
#unid_medida=>$unid_leitura\n"    

    mk::file open db medidor_multiparametro.db    
    
    mk::row append db.medida id_cad_med $id_cad_med parametro $parametro \
	tempo $tempo unid_tempo $unidade_tempo leitura $leitura unid_leitura $unid_leitura
    
    mk::file commit db
    
    mk::file close db

}


###########################################################################
#Procedimento para alertar o usuário antes da execução de uma ação
###########################################################################


proc msg_alerta { janela mensagem } {
     
    set resposta [tk_dialog $janela.alerta "Atenção!"  \
		"$mensagem"  info 0 "Continuar Exp." "Parar Exp."]
    return $resposta
	   
}


###########################################################################
#Procedimento para exibir o conteúdo do banco de dados em um listbox
###########################################################################


proc consultar_cadastro {} {
    
    global fonte_media
    set c_db .cdb
    catch {destroy $c_db}
    toplevel $c_db
    wm title $c_db "Banco de Dados dos Experimentos"
    wm iconname $c_db "Banco de Dados"
    
    frame $c_db.frame -borderwidth 10
    pack $c_db.frame -side top -expand yes -fill y
    
    scrollbar $c_db.frame.yscroll -command "$c_db.frame.lista yview"
    scrollbar $c_db.frame.xscroll -orient horizontal -command "$c_db.frame.lista xview"
    listbox $c_db.frame.lista -yscroll "$c_db.frame.yscroll set" \
	-xscroll "$c_db.frame.xscroll set" -width 130 -bg white \
	-height 25 -setgrid 1
    
    grid $c_db.frame.lista -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
    grid $c_db.frame.yscroll -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $c_db.frame.xscroll -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
    
    frame $c_db.botoes
    pack $c_db.botoes -side bottom -fill x
    button $c_db.botoes.fechar -font $fonte_media -text " Fechar " -command "destroy $c_db"
    button $c_db.botoes.remover -font $fonte_media -text " Remover" 

#Falta incluir a opção -command [list remover_registro $c_db.frame.lista ]

    button $c_db.botoes.relatorio -font $fonte_media -text "Relatório" \
	-command [list gerar_relatorio $c_db.frame.lista]
#button $c_db.botoes.grafico -text "Gráfico" -command [list gerar_grafico $c_db.frame.lista ]
    button $c_db.botoes.exportar -font $fonte_media -text "Exportar" 

#Falta incluir a opção -command exportar_cadastro
    
    pack $c_db.botoes.relatorio $c_db.botoes.exportar $c_db.botoes.remover $c_db.botoes.fechar -side left -expand 1 -fill both
    
    
    if {[file exists "medidor_multiparametro.db"]} {
	    
	    mk::file open db medidor_multiparametro.db -readonly
	    
	    mk::loop i db.cadastro {
		
		array set registro [mk::get $i]
		
		set registro(data_col) [clock format $registro(data_col) \
		    -format {%d/%m/%Y %H:%M}]
		
		$c_db.frame.lista insert 0 "$registro(id_cad_exp) \
\{Projeto: $registro(cod_proj)\} \
\{Experimento: $registro(num_exp)\} \{Amostra: $registro(local_col)\} \
\{Coleta: $registro(data_col)\} \{Parâmetros:$registro(lst_par_monit_amostra)\}"   
	    }   
	    
	    mk::file close db
	}
}


###########################################################################
#Procedimento para exibir o conteúdo do banco de dados em um listbox
###########################################################################

proc gerar_relatorio { j_l } {
    
global fonte_media 



    if {[$j_l curselection] == ""} {
	    tk_messageBox -message "Por Favor!\n Selecione um Experimento."
	    raise .cdb
	    return
	} else {
	    destroy .relatorio
	    toplevel .relatorio
	    wm title .relatorio "Relatório de Experimento"
	    wm resizable .relatorio 0 0
	    
	    set frame_1 [frame .relatorio.ctl]
	    pack $frame_1 -side top -expand yes -fill x
	    
	    set frame_2 [frame .relatorio.rel]
	    pack $frame_2 -side bottom 

	    menubutton $frame_1.arquivo -font $fonte_media \
		-text "Relatório" -relief raised \
		-menu $frame_1.arquivo.menu
	    
	    menu $frame_1.arquivo.menu

	    $frame_1.arquivo.menu add command -font $fonte_media \
		-label "Gravar" \
		-command [list gravar_arquivo_txt $frame_2.exp]
	    $frame_1.arquivo.menu add command -font $fonte_media \
		-label "Abrir" \
		-command [list abrir_arquivo $frame_2.exp]
	    $frame_1.arquivo.menu add command -font $fonte_media \
		-label "Fechar" \
		-command { destroy .relatorio }

	    menubutton $frame_1.leituras -font $fonte_media \
		-text "Leituras" -relief raised \
		-menu $frame_1.leituras.menu
	    
	    menu $frame_1.leituras.menu
	    
	    

	    set j_t [text $frame_2.exp -width 60 \
		-yscrollcommand "$frame_2.rlg set" -bg white]

	    scrollbar $frame_2.rlg -command "$frame_2.exp yview"
	    
	    pack $frame_1.arquivo $frame_1.leituras -side left -anchor w
	    pack $frame_2.rlg -side right -fill y
	    pack $frame_2.exp -side left -expand yes -fill both
	    
	    
	    puts "saída de curselection [ $j_l curselection ]"
	    puts "saída de get [ $j_l get [ $j_l curselection ] ]"
	    
#Curselection retorna a posição do item selecionado na listbox
#"get" retorna o item selecionado e "lindex" extrai apenas o primeiro campo
#que neste caso é o id do experimento no cadastro de experimentos
#Este id é usado mais abaixo no comando "select" o qual retorna
#o número da linha deste experimento
#Com esta informação o comando "get" filtra do banco de dados  todas
#as informações sobre o experimento armazena no array registro com o comando
#"array set"

	    set id_cad_exp [lindex [ $j_l get [ $j_l curselection ] ] 0]
	    puts $id_cad_exp
	    
	    
	    if {[file exists "medidor_multiparametro.db"]} {
		    
		    mk::file open db medidor_multiparametro.db -readonly
		    
		    set linha [mk::select db.cadastro id_cad_exp  $id_cad_exp]
		    
		    #puts "saída de linha => $linha"
		    
		    array set registro [mk::get db.cadastro!$linha]

		    mk::file close db		   
 
		    set registro(data_col) [clock format $registro(data_col) \
			-format {%d/%m/%Y %H:%M}]
		    
		    $j_t insert end "Projeto: $registro(cod_proj) \n\
Coordenador do Projeto: $registro(coord_proj) \n\
Número do Experimento:  $registro(num_exp) \n\
Responsável pelo Experimento: $registro(resp_exp) \n\
Local de Amostragem: $registro(local_col) \n\
Data de Amostragem: $registro(data_col) \n\
Responsável pela Amostragem: $registro(resp_col) \n\
Parâmetros Monitorados: $registro(lst_par_monit_amostra) \n\
Comentários: $registro(coment_exp)"  


#Este loop insere no menubutton "Leituras" os botões correspondentes às
#leituras realizadas e chama a rotina "exportar_leituras" passando
#como argumento o nome do parâmetro e o id do experimento na tabela
#cadastro

		    foreach { p lim_inf lim_sup } $registro(lst_par_monit_amostra) {
			
			$frame_1.leituras.menu add command \
			    -font $fonte_media \
			    -label "Exportar Leituras de $p" \
			    -command [list exportar_leituras \
			    $registro(cod_proj) $registro(num_exp) $id_cad_exp $p ]
		    }	   
		}
	}
}



proc exportar_leituras { proj exp id_cad par } {
    
    global unidade_tempo

    if {[file exists "medidor_multiparametro.db"]} {
	    
	    set tipo_arquivo {
		{"Arquivos de Leituras"  { .csv } }
		{"Todos os arquivos" * }
	    }
	    set nome_arquivo [tk_getSaveFile -filetypes $tipo_arquivo -initialdir pwd]
	    if {[catch {set id_arquivo [open $nome_arquivo w]} result] != 0} {
		    return
		}
		
		puts $id_arquivo "Parâmetro: $par\nExperimento: $exp\n\Projeto: $proj\nID(Banco_Dados): $id_cad\n"

		puts $id_arquivo "Tempo (unidade) Leitura (unidade)"
		
		mk::file open db medidor_multiparametro.db -readonly
		
		foreach reg [mk::select db.medida id_cad_med $id_cad parametro $par] {
		    
		    set dados [mk::get db.medida!$reg \
			tempo unid_tempo leitura unid_leitura]

		    set tempo [format "%.2f" [lindex $dados 0]]
		    set unid_tempo [lindex $dados 1]
		    set leitura [format "%.2f" [lindex $dados 2]]
		    set unid_leitura [lindex $dados 3]

#Os dados são armazenados com "." como sinal decimal e são convertidos
#no formato brasileiro, com  "," como sinal decimal antes de exportar
		    set tempo  [regsub {\.} $tempo {,} ]
		    set leitura [regsub {\.} $leitura {,} ]

		    puts $id_arquivo "$tempo $unid_tempo $leitura $unid_leitura"
		    
		} 
		
		close $id_arquivo		
		mk::file close db
		
	}
	
}


##############################################################################
#Procedimento para exibir as telas para os gráficos de acompanhamento
##############################################################################

proc exibir_tela_grafico {} {
    
    global par_monit
    
    destroy .graf
    toplevel .graf
    
    wm title .graf "Gráficos"
    
    wm resizable .graf 0 0
    
    if {[llength $par_monit] >= 9} {
	    set altura_graf 150 
	} else {
	    set altura_graf 300
	}
	
	foreach { par nome unidade } $par_monit {

#Label frame para acomodar o gráfico e o frame para configuração	    
	    set grafico_$par [labelframe .graf.[string tolower $par] -text $nome]
#Canvas para o gráfico
	    set canvas_$par [canvas [set grafico_$par].c -bg white -width 700 \
		-height $altura_graf]
#Frame para configuração
	    set conf_graf_$par [frame [set grafico_$par].config]

#Botão para configuração
	    set botao_conf_graf_$par [button [set conf_graf_$par].b_conf -text "Configurar" \
		-command [list config_grafico $par]]
		

#Botão para apagar o conteúdo do gráfico e refazer os eixos
#com chamada para os procedimentos apagar_grafico e criar_grafico
#Tentei acrescentar o comando config_grafico mas não adiantou pois
#o comando criar_grafico redefine todos os limites de X e Y para os
#valores originais
	    set botao_apagar_graf_$par [button [set conf_graf_$par].b_apagar -text "Reiniciar" \
		-command "[list apagar_grafico [set canvas_$par]]; \
		[list criar_grafico $par $unidade [set canvas_$par]]"]
	    
#Frames para as opções de configuração dos limites dos eixos X e Y
	    set frame_x_max_$par [frame [set conf_graf_$par].x_max]
	    set frame_x_min_$par [frame [set conf_graf_$par].x_min]
	    set frame_x_int_$par [frame [set conf_graf_$par].x_int]
	    set frame_y_max_$par [frame [set conf_graf_$par].y_max]
	    set frame_y_min_$par [frame [set conf_graf_$par].y_min]
	    set frame_y_int_$par [frame [set conf_graf_$par].y_int]

#Labels para identificar as entradas de configuração
	    set rot_x_max_$par [label [set frame_x_max_$par].rot -justify left \
		-text "Tempo final" -anchor w]
	    set rot_x_min_$par [label [set frame_x_min_$par].rot -justify left \
		-text "Tempo inicial" -anchor w]
	    set rot_x_int_$par [label [set frame_x_int_$par].rot -justify left \
		-text "Int. tempo" -anchor w]
	    set rot_y_max_$par [label [set frame_y_max_$par].rot -justify left \
		-text "[lindex $unidade end] máx." -anchor w]
	    set rot_y_min_$par [label [set frame_y_min_$par].rot -justify left \
		-text "[lindex $unidade end] mín." -anchor w]
	    set rot_y_int_$par [label [set frame_y_int_$par].rot -justify left \
		-text "Int. [lindex $unidade end]" -anchor w]

#Entradas para configurar os limites dos eixos
	    set ent_x_min_$par [entry [set frame_x_min_$par].ent -width 5 -bg white \
		-textvariable x_min_$par]	    
	    set ent_x_max_$par [entry [set frame_x_max_$par].ent -width 5 -bg white \
		-textvariable x_max_$par]
	    set ent_x_int_$par [entry [set frame_x_int_$par].ent -width 5 -bg white \
		-textvariable x_int_$par]
	    set ent_y_min_$par [entry [set frame_y_min_$par].ent -width 5 -bg white \
		-textvariable y_min_$par]	    
	    set ent_y_max_$par [entry [set frame_y_max_$par].ent -width 5 -bg white \
		-textvariable y_max_$par]
	    set ent_y_int_$par [entry [set frame_y_int_$par].ent -width 5 -bg white \
		-textvariable y_int_$par]

	    pack [set rot_x_min_$par] [set ent_x_min_$par] -side right \
		-expand yes -fill x -anchor w
	    pack [set rot_x_max_$par] [set ent_x_max_$par] -side right \
		-expand yes -fill x -anchor w
	    pack [set rot_x_int_$par] [set ent_x_int_$par] -side right \
		-expand yes -fill x -anchor w
	    pack [set rot_y_min_$par] [set ent_y_min_$par] -side right \
		-expand yes -fill x -anchor w
	    pack [set rot_y_max_$par] [set ent_y_max_$par] -side right \
		-expand yes -fill x -anchor w
	    pack [set rot_y_int_$par] [set ent_y_int_$par] -side right \
		-expand yes -fill x -anchor w

	    pack [set frame_x_min_$par] -expand yes -fill both
	    pack [set frame_x_max_$par] -expand yes -fill both
	    pack [set frame_x_int_$par] -expand yes -fill both
	    pack [set frame_y_min_$par] -expand yes -fill both
	    pack [set frame_y_max_$par] -expand yes -fill both
	    pack [set frame_y_int_$par] -expand yes -fill both

	    pack [set botao_apagar_graf_$par] -side bottom -expand yes -fill x	   

	    pack [set botao_conf_graf_$par] -side bottom -expand yes -fill x
	   
	    pack [set conf_graf_$par] [set canvas_$par]  -side left -expand yes -fill both

	    pack [set grafico_$par]
	    
	    criar_grafico $par $unidade [set canvas_$par]
	    
	}
	
}

##############################################################################
#Escalas default para alguns parâmetros
#Cond (uS/cm) 100 <-> 10000
#Resistividade (R = 1/C) (Mohmxcm) 0.0001 <->  0.01
#Cond (mg/L) TDS 100 <-> 1000 (Água potável portaria 518)
#Cond (ppt) TDS 100000 <-> 1000000 
#ISE (mV) -1000 <-> +1000
#ISE (RmV) -1000 <-> +1000
#OD (mg/L) 0 <-> 10
#OD (% sat) 0 <-> 100
##############################################################################

##############################################################################
#A variável unidade é uma lista com dois elementos 
#onde o segundo elemento é a unidade usada
#ymax_$parametro é o valor máximo do eixo y para o respectivo parâmetro
#ymin_$parametro é o valor mínimo do eixo y para o respectivo parâmetro
#int_$parametro é o intervalo
#g_$parametro é o gráfico do respectivo parâmetro e será armazenado no
#array global grafico
##############################################################################

proc criar_grafico { parametro unidade tela } {
    
global grafico
global x_min_$parametro x_max_$parametro x_int_$parametro 
global y_min_$parametro y_max_$parametro y_int_$parametro

#  puts "Criar gráfico para $parametro com [lindex $unidade 0] em [lindex $unidade end] no canvas $tela"
    
set x_min_$parametro 0
set x_max_$parametro 60
set x_int_$parametro 5

    switch [lindex $unidade end] \
	"uS/cm" {puts  [set y_min_$parametro 100]; \
	puts [set y_max_$parametro 10000]; \
	puts [set y_int_$parametro 1000] } \
	"Mohmxcm" {puts  [set y_min_$parametro 0.0001]; \
	puts [set y_max_$parametro 0.01]; \
	puts [set y_int_$parametro 0.001] } \
	"mg/L" {if {$parametro == "COND"} { \
	puts  [set y_min_$parametro 100]; \
	puts [set y_max_$parametro 1000]; \
	puts [set y_int_$parametro 200] } \
	else \
	{if {$parametro == "OD"} { \
	puts  [set y_min_$parametro 0]; \
	puts [set y_max_$parametro 10]; \
	puts [set y_int_$parametro 2] } } } \
	"ppt" { puts  [set y_min_$parametro 100000]; \
	puts [set y_max_$parametro 1000000]; \
	puts [set y_int_$parametro 5000] } \
	"pH" { puts  [set y_min_$parametro 0]; \
	puts [set y_max_$parametro 14]; \
	puts [set y_int_$parametro 2] } \
	"mV" { puts  [set y_min_$parametro -1000]; \
	puts [set y_max_$parametro 1000]; \
	puts [set y_int_$parametro 400] } \
	"RmV" { puts  [set y_min_$parametro -1000]; \
	puts [set y_max_$parametro 1000]; \
	puts [set y_int_$parametro 200] } \
	"ppb" { puts  [set y_min_$parametro 0]; \
	puts [set y_max_$parametro 1000]; \
	puts [set y_int_$parametro 100] } \
	"% sat" { puts  [set y_min_$parametro 0]; \
	puts [set y_max_$parametro 100]; \
	puts [set y_int_$parametro 20] } \
	"C" { puts  [set y_min_$parametro 10]; \
	puts [set y_max_$parametro 50]; \
	puts [set y_int_$parametro 10] } \
	"default" { set y_min_$parametro 0; set y_max_$parametro 10; set y_int_$parametro 2 }



#Tive que usar o comando eval pois o ::Plotchart::createXYPlot
#aceita apenas valores numéricos

set cria_g "set g_$parametro \[::Plotchart::createXYPlot $tela \
{[set x_min_$parametro] [set x_max_$parametro] [set x_int_$parametro]} \
{[set y_min_$parametro] [set y_max_$parametro] [set y_int_$parametro]}\]"

eval $cria_g

set grafico(g_$parametro) [set g_$parametro]

$grafico(g_$parametro) xtext "Tempo (min)"
$grafico(g_$parametro) ytext "[lindex $unidade end]"

#$s yconfig -format "%12.2e"

}

##############################################################################
#Procedimento para reconfigurar os limites do gráfico
#o comando rescale aceita somente valores numéricos, por isso a necessidade
#de "montar" o comando dentro da variável conf_g e em seguida chamar 
#o interpretador para executar o conteúdo desta variável com o comando "eval"
##############################################################################

proc config_grafico { parametro } {

global grafico 
global x_min_$parametro x_max_$parametro x_int_$parametro 
global y_min_$parametro y_max_$parametro y_int_$parametro

puts "Configurando gráfico [set x_min_$parametro] [set x_max_$parametro] [set x_int_$parametro]"
puts "Configurando gráfico [set y_min_$parametro] [set y_max_$parametro] [set y_int_$parametro]"

set x_min_$parametro [regsub {\,} [set x_min_$parametro] {.} ]
set x_max_$parametro [regsub {\,} [set x_max_$parametro] {.} ]
set x_int_$parametro [regsub {\,} [set x_int_$parametro] {.} ]

set y_min_$parametro [regsub {\,} [set y_min_$parametro] {.} ]
set y_max_$parametro [regsub {\,} [set y_max_$parametro] {.} ]
set y_int_$parametro [regsub {\,} [set y_int_$parametro] {.} ]


set conf_g "\$grafico(g_\$parametro) rescale \
{ [set x_min_$parametro] [set x_max_$parametro] [set x_int_$parametro] } \
{ [set y_min_$parametro] [set y_max_$parametro] [set y_int_$parametro] }"

eval $conf_g

}

##############################################################################
#Procedimento para exibir no gráfico a medida realizada
#Este procedimento usa o objeto da Classe medida e está sendo chamado pelo
#procedimento iniciar_medida
#Pensei em qual termo usar para o nome deste procedimento e ao invés de usar
#o "anglicismo" "plotar" escolhi o "hispanismo" "graficar". Pois o verbo graficar 
#existe em espanhol e é, segundo o dicionário da Real Academia Espanhola, 
#usado em Cuba, Argentina, Chile, Salvador e Uruguai; significa 
#"representar mediante figuras e símbolos" 
#(Fonte: http://www.ciberduvidas.pt/pergunta.php?id=28036
#O nome do gráfico fica armazenado no array global grafico e indexado
#com g_$parametro
#Usamos o conteúdo da variável global num_exp para permitir a sobreposição
#dos gráficos de diferentes experimentos em um mesmo gráfico
##############################################################################

#*******************************************************************************
#Seria interessante se o número do experimento (num_exp) fosse um atributo do
# objeto da Classe medida
#*******************************************************************************

proc graficar_medida { info_medida } {
    
    global grafico num_exp num_exp_ant simbolo exibir_legenda
    global unidade_tempo
    
    set parametro [$info_medida get_parametro_medido]
    
#Converter o valor do tempo para a unidade min para se ajustar à escala
#predefinida de minutos

    set tempo  [lindex [lindex [$info_medida get_leituras] 0 ] 0 ]
    
#Testes condicionais para ajustar os valores de tempo para a escala de minutos

    if { $unidade_tempo == "h" } {
	    
	    set tempo [ expr $tempo * 60.0]
	    
	}
	
	if { $unidade_tempo == "s" } {
		
		set tempo [ expr $tempo / 60.0 ]

	    }
    
    set leitura [lindex [lindex [$info_medida get_leituras] 0] 1 ]

#Esse primeiro teste verifica a existência da variável num_exp_ant
#(número do experimento anterior) que é definida dentro do procedimento
#parar_experimento.
#Na primeira execução de um experimento usa-se o símbolo plus mas
#a partir dos experimentos seguintes os experimentos são alternados entre 
#as 8 possíveis 
    
    if {![info exists num_exp_ant]} {

	    set simbolo plus
	    
	    $grafico(g_$parametro) dataconfig $num_exp -type symbol
	    
	    $grafico(g_$parametro) dataconfig $num_exp -symbol $simbolo
	    
	    $grafico(g_$parametro) plot $num_exp $tempo $leitura
	    
	    if { ![info exists exibir_legenda] } {	    

		    $grafico(g_$parametro) legend $num_exp "Exp. $num_exp"

		    set exibir_legenda 0

		}
		
	    

	    return	
	} 
	
	$grafico(g_$parametro) dataconfig $num_exp -type symbol
	
	$grafico(g_$parametro) dataconfig $num_exp -symbol $simbolo
	
	$grafico(g_$parametro) plot $num_exp $tempo $leitura
	
	if { $exibir_legenda } {

	$grafico(g_$parametro) legend $num_exp "Exp. $num_exp"

	set exibir_legenda 0

	}
}

#############################################################################
#Procedimento chamado ao final de cada experimento para mudar
#o símbolo usado nos gráficos
#Foi a maneira que eu encontrei para disparar essa alteração
#############################################################################

proc mudar_simbolo {} {
    
    global simbolo exibir_legenda
    
    set lista_simbolos {plus cross circle up down dot upfilled downfilled}
    
    set indice [lsearch -exact $lista_simbolos $simbolo]
    
    if {$indice == 7} { set indice 0 } else { incr indice }
	
	set simbolo [lindex $lista_simbolos $indice]
	
	set exibir_legenda 1
}

proc apagar_grafico { tela } {

$tela delete all

}


tela_inicial


##############################################################################
#Chama a tela de configuração e passa o nome do frame da tela de monitoramento
#O uso desse argumento foi apenas para permitir a navegação da tela de 
#configuração para a tela de monitoramento e vice-versa
##############################################################################


#tela_config .janela
