# Started using TclOO package for defining classes
# to control the aeration system, sludge addition and pH control

puts "Módulo [info script] carregado!"

#Loads the package TclOO
package require TclOO

#Create the var equipments_available to execute the control actions
#these data should be stored in a config file
#The name of equipment Ex: air_pump_I "MUST" be present as a namespace
#in the driver boyu_s_2000a and "MUST" be the same name of equipment
#in the state_transition_table

dict set equipments_available air_pump_I par OD
dict set equipments_available air_pump_I brand "BOYU"
dict set equipments_available air_pump_I model "S_2000A"
dict set equipments_available air_pump_I com_port "/dev/arduino"
dict set equipments_available air_pump_I driver "boyu_s_2000a"
dict set equipments_available air_pump_II par OD
dict set equipments_available air_pump_II brand "BOYU"
dict set equipments_available air_pump_II model "S_2000A"
dict set equipments_available air_pump_II com_port "/dev/arduino"
dict set equipments_available air_pump_II driver "boyu_s_2000a"
dict set equipments_available heater par TEMP
dict set equipments_available heater brand "FAME"
dict set equipments_available heater model "BIO"
dict set equipments_available heater com_port "/dev/arduino"
dict set equipments_available heater driver "fame_bio"

#Add a directory to the beginning of the list of directories 
#to be searched for a module. 

# Add the current folder to the search path if it
# is not already in the set of directories to be searched.

puts "path for modules before pwd [::tcl::tm::list]"
if {[lsearch [::tcl::tm::path list] [pwd]] < 0} {
  ::tcl::tm::path add [pwd]
}

::tcl::tm::path add lib_equipment/

puts "path for modules after pwd and lib_equipment [::tcl::tm::list]"

#global variable state_channel to work as a semaphore to control the access
#of equipment objects to the serial port

set state_channel 1


#Procedure to get the name(s) of equipment(s) related for a parameter

proc getNameEquipment { parameter } {

    global equipments_available
    
    set list_equip {}
    
    dict for { name info } $equipments_available {
	
	dict with info {
	    
	    if { $par == $parameter } {
		puts "getNameEquipment name:$name info:$info"
		lappend list_equip $name
	    }
	    
	}
	
    }
    
    puts "getNameEquipment list_equip:$list_equip"
    return $list_equip
}


#Proc to get informations about the equipment used to control a parameter

proc getInfoEquipment { name_equip } {
    
    global equipments_available

        
    dict for { name info } $equipments_available {

	dict with info {

	    if { $name == $name_equip } { return $info }	    
	    
	}
    }
}


#The structure of the dict var “state_transition_table” for respirometry (OD)

#dict set [name_var][parameter][present_state][input][next_state]"[equipment][action]"

#dict set state_transition_table OD S0 I0 S0 {air_pump_I on}
#dict set state_transition_table OD S0 I1 S1 {air_pump_I on}
#dict set state_transition_table OD S0 I2 S2 {air_pump_I off}
#dict set state_transition_table OD S1 I0 S0 {air_pump_I on}
#dict set state_transition_table OD S1 I1 S1 {air_pump_I ig}
#dict set state_transition_table OD S1 I2 S2 {air_pump_I off}
#dict set state_transition_table OD S2 I0 S0 {air_pump_I on}
#dict set state_transition_table OD S2 I1 S1 {air_pump_I off}
#dict set state_transition_table OD S2 I2 S2 {air_pump_I off}

#Example two commands for two different equipments for some states (S0)

dict set state_transition_table OD S0 I0 S0 {air_pump_I on air_pump_II on}
dict set state_transition_table OD S0 I1 S1 {air_pump_I on air_pump_II off}
dict set state_transition_table OD S0 I2 S2 {air_pump_I off}
dict set state_transition_table OD S1 I0 S0 {air_pump_I on air_pump_II on}
dict set state_transition_table OD S1 I1 S1 {air_pump_I ig}
dict set state_transition_table OD S1 I2 S2 {air_pump_I off}
dict set state_transition_table OD S2 I0 S0 {air_pump_I on air_pump_II on}
dict set state_transition_table OD S2 I1 S1 {air_pump_I off}
dict set state_transition_table OD S2 I2 S2 {air_pump_I off}

dict set state_transition_table TEMP S0 I0 S0 {heater on}
dict set state_transition_table TEMP S0 I1 S1 {heater off}
dict set state_transition_table TEMP S0 I2 S2 {heater off}
dict set state_transition_table TEMP S1 I0 S0 {heater on}
dict set state_transition_table TEMP S1 I1 S1 {heater off}
dict set state_transition_table TEMP S1 I2 S2 {heater off}
dict set state_transition_table TEMP S2 I0 S0 {heater on}
dict set state_transition_table TEMP S2 I1 S1 {heater off}
dict set state_transition_table TEMP S2 I2 S2 {heater off}

#To be implemented
#Procedure to classify the input (readout)
#lcl - lower control limit
#ucl - upper control limit
#lal - lower alert limt
#ual - upper alert limit

proc classifyInput { input lcl ucl {lal none} {ual none} } {

}

proc getInfoState { parameter present_state input } {

    global state_transition_table

#puts "getInfoState parameter:$parameter present_state:$present_state input:$input"

#    set test_parameter [catch {dict get $state_transition_table $parameter}]

#Checks if there is the parameter $parameter in the state transition table

    if {![catch {dict get $state_transition_table $parameter}]} {

    set next_state_action [dict get [dict get [dict get $state_transition_table $parameter] $present_state] $input]

#puts "getInfoState next_state_action:$next_state_action"

return $next_state_action

    }

}

#=================================================================

#Create the class State

oo::class create state {

    variable parameter par_max par_min list_obj_equip present_state next_start

    constructor {par max min} {
	set parameter $par
	set par_max $max
	set par_min $min
    }
    
    method setEquip { l_e } {
    set list_obj_equip $l_e
    }
    
    method getEquip {} {

	return $list_obj_equip

    }

    method setState {s} {

	set present_state $s

    }

    method getState {} {

	return $present_state

    }

    method checkState {} {
	
	#Check if there are readout available in global variable readings. 
	#If not, returns and schedule new start method
	
	if {[catch {set readout [getReading $parameter]}]} {
	    puts "checkState -> No readout"
	    [self] start
	    return
	}
	
	set readout [lindex $readout 1]
	
#puts "checkState par:$parameter readout:$readout par_min:$par_min par_max:$par_max"
	
	if { $readout <= $par_min } {
	    set input I0
	    #	    puts "checkState $parameter I0"
	} elseif { $readout > $par_min && $readout < $par_max } {
	    set input I1
	    #	    puts "checkState $parameter I1"
	} elseif {$readout >= $par_max} {
	    set input I2
	    #	    puts "checkState $parameter I2"
	}
	
	set next_state_equip_action [getInfoState $parameter $present_state $input]
	
#	puts "checkState [getInfoState $parameter $present_state $input]"
	
	[self] setState [lindex $next_state_equip_action 0]
	
	#The variable next_state_equip_action is a list of list
	#the first element is the "next_state" and the second element is a list
	#of pairs "equipment action"
	#So we use a foreach loop to interact on the elements of list (equipment action)
	#to call the method sendCmd for each instance of equipment
	
	#	$equipment sendCmd [lindex [lindex $next_state_equip_action 1] 1]
	
	set list_equip_action [lindex $next_state_equip_action 1]
	
#puts "checkState of [self] list_equip_action:$list_equip_action"
	
	#puts "checkState of [self] list of objects equipment:[[self] getEquip]"
	
	#puts "checkState of [self] list_obj_equip:$list_obj_equip"
	
	foreach {equip act} $list_equip_action {
	    
#	    puts "checkState of [self] equip:$equip act:$act"
	    
	    foreach obj_equip $list_obj_equip {
		
		if {[$obj_equip getName] == $equip} {
#		    puts "[self]-> scheduling $obj_equip to execute $act"

		    after 1 [list $obj_equip sendCmd $act]
		    
		}
		
	    }
	    
	}	
	
	[self] start
    }
    
    method start {} {
	set next_start [after 2000 [list [self] checkState]]
    }

    method stop {} {
	after cancel $next_start
    }
}

#Procedure to open a channel

proc openChannel {port} {

    global id_port
    
    if {![info exists id_port]} {set id_port 0}
    
    if {[chan names $id_port] == ""}  {
	
	set baud 9600
	set parity n
	set data_bit 8
	set stop_bit 1
	
	set result_1 [ catch { set id_port [open $port r+] } ]
	
	set result_2 [catch {fconfigure $id_port -mode "$baud,$parity,$data_bit,$stop_bit" }]
	
	if { ($result_1 != 0) || ($result_2 != 0) } {
	    set id_port "SERIAL_ERRO_OPEN"
#	    puts "ERROR - Opening channel:$id_port"
	    return $id_port
	} else {
	    fconfigure $id_port -blocking 0 -buffering none
#	    puts "Opening channel:$id_port"
	    return $id_port
	}
    } else {
	puts "Channel:$id_port already open"
	return $id_port
    }
}


proc closeChannel {} {

global id_port

catch {close $id_port}

}


#Procedure to initialize state_channel

proc initStateChannel {} {

global state_channel

set state_channel 1

}

#Procedure to request access to the channel 

proc requestKeyChannel {} {
    
    global state_channel
    
#puts "requesting key state_channel:$state_channel"

    if {$state_channel} {
	
	set state_channel 0
	
#	puts "LOCKING CHANNEL"

	return 1

    } else {

	return 0
    }
}

proc unlockChannel {} {

    global state_channel
    
    if {!($state_channel)} {
	set state_channel 1
#	puts "UNLOCKING CHANNEL"
    } else {
#	puts "ALREADY UNLOCKED"
    }
}

#=================================================================

#Create the class equipment

oo::class create equipment {
    
    variable parameter name brand model communication_port channel key_channel driver id_after repeat
    
    constructor {par b m} {
	
	set parameter $par
	set brand $b
	set model $m
	set repeat 0
	
    }
    
    method setName {n} {
	set name $n
    }

    method getName {} {
	return $name
    }

    method setCommunicationPort { port } {
	set communication_port $port
    }

    method getCommunicationPort {} {
	return $communication_port
    }

    method setChannel {c} {
	set channel $c
    }

    method getChannel {} {
	return $channel
    }

    method setKeyChannel {k} {
	set key_channel $k
    }

    method getKeyChannel {} {
	return $key_channel
    }

    method setDriver {d} {
	set driver $d
    }

    method getDriver {} {
	return $driver
    }

    method sendCmd {c} {
	
#	puts "[self] BEFORE request key_channel key_channel:[[self] getKeyChannel]"
	
	if {[info object isa object [self]]} {
	    [self] setKeyChannel [requestKeyChannel]
	} else {
	    return 0
	}
	
#	puts "[self] AFTER request key_channel key_channel:[[self] getKeyChannel]"
	
	if {[[self] getKeyChannel]} {
	    #if key_channel == 1 send the command
	    
	    #puts "[self] with key sending command $c to channel:$channel"
	    
	    #call a procedure stored in var c which is defined in a namespace whose
	    #the name is the same as the atribute "name" of this object
	    #This "name" is not the name of object but the name of equipment used by
	    #this object
	    
	    #to check if object exists until now 
	    #or was destroyed by the command stop
	    
	    if {[info object isa object [self]]} {
		#24/03/2014
		#As I observed the serial port (ttyUSB0) was closed suddenly I decided to include an if test
		
#		puts "[self] sendCmd - \[\[self\] getCommunicationPort\]:[[self] getCommunicationPort]"
#		puts "[self] sendCmd - \[file exist \[\[self\] getCommunicationPort\]\]:[file exist [[self] getCommunicationPort]]"
		if {[file exist [[self] getCommunicationPort]]} {
		    
#		    puts "[self] - device [[self] getCommunicationPort] exist, sending command $c, by channel:$channel, to [[self] getName]"
		    #In case the change of the number of the device the channel is unavailable		    
		    if {![[[self] getName]::$c $channel]} {
			puts "[info_time] [self] - Error from the module Arduino, closing and reopening the channel and returning 0"
			closeChannel
			[self] setChannel [openChannel [[self] getCommunicationPort]]
			[self] setKeyChannel 0
			unlockChannel
			return 0
			#puts "[info_time] [self] - Error from the module Arduino - Sending $c to $channel once more"
			#[[self] getName]::$c $channel
		    }
		    
		} else {
		    puts "[info_time] [self] device [[self] getCommunicationPort] does not exist"
		    puts "[info_time] [self] - closing and reopening the channel"
		    closeChannel
		    [self] setChannel [openChannel [[self] getCommunicationPort]]
		    
		    if { $channel != "SERIAL_ERRO_OPEN" } {
			puts "[self] - key_channel:[[self] getKeyChannel]"
			[[self] getName]::$c $channel
			puts "[self] - unlocking channel:$channel after send the command $c to [[self] getName] and returning 1"
			[self] setKeyChannel 0
			unlockChannel
			return 1
			#stopControl "[self] device [[self] getCommunicationPort] does not exist"
		    } else {
			puts "[info_time] [self] - unlocking channel:$channel after NOT send the command $c to [[self] getName] and returning 0"
			[self] setKeyChannel 0
			unlockChannel
			return 0
		    }
		}
		
	    } else {
		return 0
	    }
	    
	    
	    #to check if object exists until now 
	    #or was destroyed by the command stop
	    #set key_channel 0
	    if {[info object isa object [self]]} {
		#puts "Object [self] exists"
		[self] setKeyChannel 0
	    } else {
		return 0
	    }
	    
	    #unlock the channel
#	    puts "[self] - unlocking the Channel"
	    unlockChannel
#	    puts "[self] command $c sent to [[self] getName] in attempt $repeat"
	    
	    set repeat 0
	    
	} else {
#	    puts "[self] command $c NOT sent to [[self] getName] in the attempt $repeat"
	    
	    if {$repeat >= 5} { 
		puts "[info_time] [self] command $c NOT sent to [[self] getName] after 5 attempts"
		set repeat 0
		return 0
	    }
	    
	    #schedule to make another requests
	    incr repeat
	    
	    set id_after [after 100 [list [self] sendCmd $c]]
	    
	}
    }

	
	method start {} {
	puts "[self] opening [[self] getCommunicationPort]"
    }
    
    method stop {} {
	
	#Check the existence of variable id_after which will be created
	#in the scheduling of another request
	
	if {[info exists id_after]} { 
	    after cancel $id_after
	}
	
	#	puts "Safe shutdown: [self] turning off [[self] getName]"
	if {[info exists channel]} {
	    [[self] getName]::off $channel
	} else {
	    puts "ERROR - [self] stop"
	}
    }
}


#creates the variable "readings" of the dict type 
#that will store the current readings of the parameters 
#and will be used by the controller objects to control the variables
#through the control of related objects of the class "equipment"

set readings [dict create]
#t:time r:reading ut:unit_time (h|m|s) ur:unit_reading
#dict set readings pH t 1
#dict set readings pH r 1
#dict set readings pH ur ""
#dict set readings pH ut s
#dict set readings OD t 1
#dict set readings OD r 1
#dict set readings OD ur mg/L
#dict set readings OD ut s
#Return of values
#set t [dict get $readings pH t]
#set r [dict get $readings pH r]
#Unset remove key "t" and its corresponding value "1"
#dict unset readings pH t

#Procedure to update the variable dict "readings"

proc updateReadings { parameter t r } {

    global readings
    
    dict set readings $parameter t $t
    dict set readings $parameter r $r
    
#    puts "updateReadings t:[dict get $readings $parameter t]"
#    puts "updateReadings r:[dict get $readings $parameter r]"

}

proc getReading { parameter } {

global readings
    
    set t [dict get $readings $parameter t]
    set r [dict get $readings $parameter r]

    return [list $t $r]
    
}

#Procedure to create the objects of the class State and Equipment
#name_of_class create name_of_object
#or
#set variable_to_store_name_object [name_of_class new var_to_constructor]
#

proc configControl { list_par_cont } {
    
    global readings 

    set list_obj_equip {}
    
 #   puts "List of parameters to be controlled:$list_par_cont"
    
    foreach {par max min} $list_par_cont {

#Create the instances of class Equipment
	
	set list_name_equipment [getNameEquipment $par]
	
#	puts "configControl list_name_equipment:$list_name_equipment"

	
	foreach n_e $list_name_equipment {
	    
	    set info_equipment [getInfoEquipment $n_e]
	    
#	    puts "configControl info_equipment:$info_equipment"

	    set obj_equipment [equipment new $par \
				   [dict get $info_equipment brand] \
				   [dict get $info_equipment model]]
	    $obj_equipment setName $n_e
	    $obj_equipment setCommunicationPort [dict get $info_equipment com_port]
	    set c [openChannel [$obj_equipment getCommunicationPort]]

#Check if the Arduino board is connected
#The procedure configControl is called by the procedure iniciar_experimento from monitor.tcl
#so if any problem in openChannel command this procedure returns 0 and stop the start of
#experiment
	    if { $c == "SERIAL_ERRO_OPEN" } {
		puts "configControl -> ERROR - SERIAL_ERRO_OPEN"
		stopControl
		return 0
	    }
#puts "configControl channel:$c"

	    $obj_equipment setChannel $c
	    
#Initialize de variable key_channel of equipment with 0
	    $obj_equipment setKeyChannel 0

#Load the driver to the instance of equipment

	puts [package require [dict get $info_equipment driver]]

#	[$obj_equipment getName]::on
#	[$obj_equipment getName]::off

	    lappend list_obj_equip $obj_equipment

#	    puts "configControl list_obj_equip:$list_obj_equip"

	}
#	equipment create $name_obj_equipment $par \
#	    [dict get $info_equipment brand] \
#	    [dict get $info_equipment model]

	
	
#file join -> Une nome de arquivo e diretórios
#usando o separador correto para o sistema operacional.
#	set driver_equipment [dict get $info_equipment driver]
#	source [file join lib_equipment $driver_equipment]



#Create the instances of class State

	state create state_$par $par $max $min
	
	state_$par setState S0

	state_$par setEquip $list_obj_equip

#	puts "configControl list_obj_equip:$list_obj_equip of state_$par"

	set list_obj_equip {}
	
    }
    
     
    set list_obj_equipment [info class instances equipment]

#    puts "configControl list of all equipments:$list_obj_equipment"

    set list_obj_state [info class instances state]
    
#    puts "configControl list of all states:$list_obj_state"

    startControl $list_obj_state

#Return 1 to the caller iniciar_experiment from monitor.tcl
    return 1
}

proc startControl { list_obj_state } {

initStateChannel

#puts "startControl list_obj_state:$list_obj_state"

    foreach obj_state $list_obj_state {
	puts "startControl Iniciando $obj_state"
	$obj_state start
    }
    
}

#Procedure stopControl 
#send command stop to all intances of class state and equipment
#and destroy all instances of class state and equipment

proc stopControl { {msg Normal} } {

global estado_experimento

    foreach obj_state [info class instances state] {
	
	puts "stopControl $obj_state"
	
	catch {$obj_state stop}
	catch {$obj_state destroy}
	
    }
    
    foreach obj_equip [info class instances equipment] {

	puts "stopControl $obj_equip"

	catch {$obj_equip stop}
	catch {$obj_equip destroy}

    }
      
    closeChannel  

#If the procedure is called from any object of objcont module then
#this if is used to stop all experiment
#but because this we have to include the check o existence of the objects
#or just catch the return of the commands obj_state (obj_equip) stop and destroy

    if { $estado_experimento } { 
	parar_experimento $msg
    }

}


#Procedure to find differences in two lists

proc diffList { l1 l2 } {
    set diff_list ""

    foreach elem $l1 {
	if {$elem ni $l2} {lappend diff_list $elem}
    }

    foreach elem $l2 {
	if {$elem ni $l1} {lappend diff_list $elem}
    }

    return $diff_list 

}

proc info_time {} {

    set now [clock seconds]
    set date [clock format $now]
    return [format "\[%s\]" $date]
}